from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class EKSControlPlaneLogging(BaseResourceCheck):
    def __init__(self):
        name = "Ensure Amazon EKS control plane logging enabled for all log types"
        id = "CKV_AWS_37"
        supported_resources = ['aws_eks_cluster']
        categories = [CheckCategories.KUBERNETES]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        """
            Looks for enabled_cluster_log_types at aws_eks_cluster:
            https://www.terraform.io/docs/providers/aws/r/eks_cluster.html
        :param conf: aws_eks_cluster configuration
        :return: <CheckResult>
        """
        if "enabled_cluster_log_types" in conf.keys():
            log_types = ["api", "audit", "authenticator", "controllerManager", "scheduler"]

            if conf["enabled_cluster_log_types"][0] == None:
                return CheckResult.FAILED
            if all(elem in conf["enabled_cluster_log_types"][0] for elem in log_types):
                return CheckResult.PASSED
            else:
                return CheckResult.FAILED
        else:
            return CheckResult.FAILED


check = EKSControlPlaneLogging()



from checkov.common.models.enums import (
    CheckCategories,
    CheckResult,
)
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class IMDSv1Disabled(BaseResourceCheck):

    def __init__(self):
        name = "Ensure Instance Metadata Service Version 1 is not enabled"
        id = "CKV_AWS_79"
        categories = [CheckCategories.GENERAL_SECURITY]
        supported_resources = ['aws_instance', 'aws_launch_template']
        super().__init__(
            name=name,
            id=id,
            categories=categories,
            supported_resources=supported_resources,
        )

    def scan_resource_conf(self, conf):
        """
        Looks for if the metadata service is disabled or requires session tokens:
            https://registry.terraform.io/providers/hashicorp/aws/latest/docs/resources/instance#metadata-options
            or
            https://registry.terraform.io/providers/hashicorp/aws/latest/docs/resources/launch_template#metadata-options

        :param conf: dict of supported resource configuration
        :return: <CheckResult>
        """
        if (
            'metadata_options' not in conf.keys()
            or
            not isinstance(conf['metadata_options'][0], dict)
        ):
            return CheckResult.FAILED

        metadata_options = conf['metadata_options'][0]
        if (
            metadata_options.get("http_tokens") == ["required"]
            or
            metadata_options.get("http_endpoint") == ["disabled"]
        ):
            return CheckResult.PASSED
        return CheckResult.FAILED


check = IMDSv1Disabled()



from checkov.terraform.context_parsers.tf_plan import parse

simple_types = [str, int, float, bool]


def _is_simple_type(obj):
    for simple_type in simple_types:
        if isinstance(obj, simple_type) or obj == None:
            return True
    return False


def _is_list_of_simple_types(l):
    if not isinstance(l, list):
        return False
    for i in l:
        if not _is_simple_type(i):
            return False
    return True


def _is_list_of_dicts(l):
    if not isinstance(l, list):
        return False
    for i in l:
        if isinstance(i, dict):
            return True
    return False


def _hclify(obj):
    ret_dict = {}
    if not isinstance(obj, dict):
        raise Exception("this method receives only dicts")
    if hasattr(obj, 'start_mark') and hasattr(obj, "end_mark"):
        obj["start_line"] = obj.start_mark.line
        obj["end_line"] = obj.end_mark.line
    for key, value in obj.items():
        if _is_simple_type(value) or _is_list_of_simple_types(value):
            ret_dict[key] = [value]

        if _is_list_of_dicts(value):
            child_list = []
            for internal_val in value:
                child_list.append(_hclify(internal_val))
            ret_dict[key] = child_list
        if isinstance(value, dict):
            child_dict = _hclify(value)
            ret_dict[key] = child_dict
    return ret_dict


def parse_tf_plan(tf_plan_file):
    """

    :type tf_plan_file: str - path to plan file
    :rtype: tf_definition dictionary
    """
    tf_defintions = {}
    tf_defintions[tf_plan_file] = {}
    tf_defintions[tf_plan_file]['resource'] = []
    template, template_lines = parse(tf_plan_file)
    for resource in template.get('planned_values', {}).get("root_module", {}).get("resources", []):
        resource_block = {}
        resource_block[resource['type']] = {}
        resource_block[resource['type']][resource.get('name', "default")] = _hclify(resource['values'])
        tf_defintions[tf_plan_file]['resource'].append(resource_block)

    return tf_defintions, template_lines



import hcl2

from checkov.terraform.context_parsers.base_parser import BaseContextParser


class ProviderContextParser(BaseContextParser):
    def __init__(self):
        definition_type = "provider"
        super().__init__(definition_type=definition_type)

    def get_entity_context_path(self, entity_block):
        entity_type = next(iter(entity_block))
        return [entity_type] + entity_block[entity_type].get('alias', ['default'])

    def _is_block_signature(self, line_num, line_tokens, entity_context_path):
        # Ignore the alias as it is not part of the signature
        is_provider = super()._is_block_signature(line_num, line_tokens, entity_context_path[0:-1])
        if not is_provider or '=' in line_tokens or line_tokens[0] != 'provider':
            # The line provider = alias is not a provider block although it has the correct words
            # Also skips comments that include words like provider and aws
            return False

        end_line = self._compute_definition_end_line(line_num)
        provider_type = entity_context_path[0]
        provider_obj = hcl2.loads("\n".join(map(lambda obj: obj[1], self.file_lines[line_num - 1:end_line if end_line > line_num else line_num])))['provider'][0]
        alias = provider_obj[provider_type].get('alias', ['default'])
        return super()._is_block_signature(line_num, line_tokens + alias, entity_context_path)


parser = ProviderContextParser()



from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class ElasticsearchDomainEnforceHTTPS(BaseResourceValueCheck):

    def __init__(self):
        name = "Ensure Elasticsearch Domain enforces HTTPS"
        id = "CKV_AWS_83"
        supported_resources = ['aws_elasticsearch_domain']
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return "domain_endpoint_options/[0]/enforce_https"


check = ElasticsearchDomainEnforceHTTPS()



import logging
import re
from abc import ABC, abstractmethod
from itertools import islice

import dpath.util

from checkov.common.comment.enum import COMMENT_REGEX
from checkov.common.models.enums import ContextCategories
from checkov.terraform.context_parsers.registry import parser_registry
from checkov.common.bridgecrew.platform_integration import bc_integration

OPEN_CURLY = '{'
CLOSE_CURLY = '}'


class BaseContextParser(ABC):
    definition_type = ""
    tf_file = ""
    file_lines = []
    context = {}

    def __init__(self, definition_type):
        self.logger = logging.getLogger("{}".format(self.__module__))
        if definition_type.upper() not in ContextCategories.__members__:
            self.logger.error("Terraform context parser type not supported yet")
            raise Exception()
        self.definition_type = definition_type
        parser_registry.register(self)

    @abstractmethod
    def get_entity_context_path(self, entity_block):
        """
        returns the entity's path in the context parser
        :param entity_block: entity definition block
        :return: list of nested entity's keys in the context parser
        """
        raise NotImplementedError

    def _is_block_signature(self, line_num, line_tokens, entity_context_path):
        """
        Determine if the given tokenized line token is the entity signature line
        :param line_num: The line number in the file
        :param line_tokens: list of line tokens
        :param entity_context_path: the entity's path in the context parser
        :return: True/False
        """
        block_type = self.get_block_type()
        return all(x in line_tokens for x in [block_type] + entity_context_path)

    @staticmethod
    def _trim_whitespaces_linebreaks(text):
        return text.strip()

    def _filter_file_lines(self):
        parsed_file_lines = [(ind, self._trim_whitespaces_linebreaks(line)) for (ind, line) in self.file_lines]
        self.filtered_lines = [(ind, line) for (ind, line) in parsed_file_lines if line]
        return self.filtered_lines

    def _read_file_lines(self):
        with(open(self.tf_file, 'r')) as file:
            file.seek(0)
            file_lines = [(ind + 1, line) for (ind, line) in
                          list(enumerate(file.readlines()))]
            return file_lines

    def _collect_skip_comments(self, definition_blocks):
        """
        Collects checkov skip comments to all definition blocks
        :param definition_blocks: parsed definition blocks
        :return: context enriched with with skipped checks per skipped entity
        """
        bc_id_mapping = bc_integration.get_id_mapping()
        parsed_file_lines = self.filtered_lines
        comments = [(line_num, {"id": re.search(COMMENT_REGEX, x).group(2),
                                "suppress_comment": re.search(COMMENT_REGEX, x).group(3)[1:] if re.search(COMMENT_REGEX,
                                                                                                          x).group(3)
                                else "No comment provided"}) for (line_num, x) in
                    parsed_file_lines if re.search(COMMENT_REGEX, x)]
        for entity_block in definition_blocks:
            skipped_checks = []
            entity_context_path = self.get_entity_context_path(entity_block)
            context_search = dpath.search(self.context, entity_context_path, yielded=True)
            for _, entity_context in context_search:
                for (skip_check_line_num, skip_check) in comments:
                    if entity_context['start_line'] < skip_check_line_num < entity_context['end_line']:
                        if bc_id_mapping and skip_check['id'] in bc_id_mapping:
                            skip_check['id'] = bc_id_mapping[skip_check['id']]
                        skipped_checks.append(skip_check)
            dpath.new(self.context, entity_context_path + ['skipped_checks'], skipped_checks)
        return self.context

    def _compute_definition_end_line(self, start_line_num):
        """ Given the code block's start line, compute the block's end line
        :param start_line_num: code block's first line number (the signature line)
        :return: the code block's last line number
        """
        parsed_file_lines = self.filtered_lines
        start_line_idx = [line_num for (line_num, _) in parsed_file_lines].index(start_line_num)
        i = 1
        end_line_num = 0
        for (line_num, line) in islice(parsed_file_lines, start_line_idx + 1, None):
            if OPEN_CURLY in line:
                i = i + 1
            if CLOSE_CURLY in line:
                i = i - 1
                if i == 0:
                    end_line_num = line_num
                    break
        return end_line_num

    def run(self, tf_file, definition_blocks, collect_skip_comments=True):
        self.tf_file = tf_file
        self.context = {}
        self.file_lines = self._read_file_lines()
        self.context = self.enrich_definition_block(definition_blocks)
        if collect_skip_comments:
            self.context = self._collect_skip_comments(definition_blocks)
        return self.context

    def get_block_type(self):
        return self.definition_type

    def enrich_definition_block(self, definition_blocks):
        """
        Enrich the context of a Terraform block
        :param definition_blocks: Terraform block, key-value dictionary
        :return: Enriched block context
        """
        parsed_file_lines = self._filter_file_lines()
        potential_block_start_lines = [(ind, line) for (ind, line) in parsed_file_lines if line.startswith(self.get_block_type())]
        for i, entity_block in enumerate(definition_blocks):
            entity_context_path = self.get_entity_context_path(entity_block)
            for line_num, line in potential_block_start_lines:
                line_tokens = [x.replace('"', "") for x in line.split()]
                if self._is_block_signature(line_num, line_tokens, entity_context_path):
                    logging.debug(f'created context for {" ".join(entity_context_path)}')
                    start_line = line_num
                    end_line = self._compute_definition_end_line(line_num)
                    dpath.new(self.context, entity_context_path + ["start_line"], start_line)
                    dpath.new(self.context, entity_context_path + ["end_line"], end_line)
                    dpath.new(self.context, entity_context_path + ["code_lines"],
                              self.file_lines[start_line - 1: end_line])
                    potential_block_start_lines.remove((line_num, line))
                    break
        return self.context
